@@ -10,7 +10,6 @@ from multiprocessing.process import BaseProcess
 from typing import Any, Optional
 from typing import cast
 
-import six
 import wandb
 from wandb.proto import wandb_internal_pb2 as pb
 from wandb.proto import wandb_telemetry_pb2 as tpb
@@ -33,7 +32,9 @@ class InterfaceShared(InterfaceBase):
     _router: Optional[MessageRouter]
 
     def __init__(
-        self, process: BaseProcess = None, process_check: bool = True,
+        self,
+        process: BaseProcess = None,
+        process_check: bool = True,
     ) -> None:
         super().__init__()
         self._process = process
@@ -76,7 +77,7 @@ class InterfaceShared(InterfaceBase):
         stats = pb.StatsRecord()
         stats.stats_type = pb.StatsRecord.StatsType.SYSTEM
         stats.timestamp.GetCurrentTime()
-        for k, v in six.iteritems(stats_dict):
+        for k, v in stats_dict.items():
             item = stats.item.add()
             item.key = k
             item.value_json = json_dumps_safer(json_friendly(v)[0])
