@@ -182,6 +182,7 @@ class SendManager:
         Currently, we're using this primarily for `sync.py`.
         """
         files_dir = os.path.join(root_dir, "files")
+        # TODO(settings) replace with wandb.Settings
         sd: SettingsDict = dict(
             files_dir=files_dir,
             root_dir=root_dir,
@@ -201,6 +202,8 @@ class SendManager:
             save_code=None,
             email=None,
             silent=None,
+            _live_policy_rate_limit=None,
+            _live_policy_wait_time=None,
         )
         settings = SettingsStatic(sd)
         record_q: "Queue[Record]" = queue.Queue()
@@ -805,9 +808,7 @@ class SendManager:
         )
         self._fs.start()
         self._pusher = FilePusher(self._api, self._fs, silent=self._settings.silent)
-        self._dir_watcher = DirWatcher(
-            self._settings, self._api, self._pusher, file_dir
-        )
+        self._dir_watcher = DirWatcher(self._settings, self._pusher, file_dir)
         logger.info(
             "run started: %s with start time %s",
             self._run.run_id,
@@ -840,7 +841,7 @@ class SendManager:
         summary_path = os.path.join(self._settings.files_dir, filenames.SUMMARY_FNAME)
         with open(summary_path, "w") as f:
             f.write(json_summary)
-        self._save_file(filenames.SUMMARY_FNAME)
+        self._save_file(interface.GlobStr(filenames.SUMMARY_FNAME))
 
     def send_stats(self, record: "Record") -> None:
         stats = record.stats
@@ -942,7 +943,9 @@ class SendManager:
         self._telemetry_obj.MergeFrom(telem)
         self._update_config()
 
-    def _save_file(self, fname: str, policy: str = "end") -> None:
+    def _save_file(
+        self, fname: interface.GlobStr, policy: "interface.PolicyName" = "end"
+    ) -> None:
         logger.info("saving file %s with policy %s", fname, policy)
         if self._dir_watcher:
             self._dir_watcher.update_policy(fname, policy)
@@ -951,7 +954,9 @@ class SendManager:
         files = record.files
         for k in files.files:
             # TODO(jhr): fix paths with directories
-            self._save_file(k.path, interface.file_enum_to_policy(k.policy))
+            self._save_file(
+                interface.GlobStr(k.path), interface.file_enum_to_policy(k.policy)
+            )
 
     def send_header(self, record: "Record") -> None:
         pass
