@@ -8,13 +8,11 @@ For more on using `wandb.init()`, including code snippets, check out our
 [guide and FAQs](https://docs.wandb.ai/guides/track/launch).
 """
 import copy
-import datetime
 import logging
 import os
 import platform
 import sys
 import tempfile
-import time
 import traceback
 from typing import Any, Dict, Optional, Sequence, Union
 
@@ -30,6 +28,7 @@ from . import wandb_login, wandb_setup
 from .backend.backend import Backend
 from .lib import filesystem, ipython, module, reporting, telemetry
 from .lib import RunDisabled, SummaryDisabled
+from .lib.deprecate import deprecate, Deprecated
 from .lib.printer import get_printer
 from .lib.proto_util import message_to_dict
 from .lib.wburls import wburls
@@ -92,6 +91,8 @@ class _WandbInit:
 
         self._init_telemetry_obj = telemetry.TelemetryRecord()
 
+        self.deprecated_features_used: Dict[str, str] = dict()
+
     def setup(self, kwargs) -> None:  # noqa: C901
         """Completes setup for `wandb.init()`.
 
@@ -136,16 +137,8 @@ class _WandbInit:
         # Start with settings from wandb library singleton
         settings: Settings = self._wl.settings.copy()
         settings_param = kwargs.pop("settings", None)
-        if settings_param is not None:
-            if isinstance(settings_param, Settings):
-                # todo: check the logic here. this _only_ comes up in tests?
-                # update settings with settings_param using whatever
-                # source each parameter has there
-                settings._apply_settings(settings_param, _logger=logger)
-            elif isinstance(settings_param, dict):
-                # if it is a mapping, update the settings with it
-                # explicitly using Source.INIT
-                settings.update(settings_param, source=Source.INIT)
+        if settings_param is not None and isinstance(settings_param, (Settings, dict)):
+            settings.update(settings_param, source=Source.INIT)
 
         self._reporter = reporting.setup_reporter(settings=settings)
 
@@ -177,19 +170,23 @@ class _WandbInit:
         # Remove parameters that are not part of settings
         init_config = kwargs.pop("config", None) or dict()
 
-        config_include_keys = kwargs.pop("config_include_keys", None)
-        config_exclude_keys = kwargs.pop("config_exclude_keys", None)
-
-        # todo: deprecate config_include_keys and config_exclude_keys
-        # if config_include_keys or config_exclude_keys:
-        #     wandb.termwarn(
-        #       "config_include_keys and config_exclude_keys are deprecated:"
-        #       " use config=wandb.helper.parse_config(config_object, include=('key',))"
-        #       " or config=wandb.helper.parse_config(config_object, exclude=('key',))"
-        #     )
+        # todo: remove this once officially deprecated
+        deprecated_kwargs = {
+            "config_include_keys": (
+                "Use `config=wandb.helper.parse_config(config_object, include=('key',))` instead."
+            ),
+            "config_exclude_keys": (
+                "Use `config=wandb.helper.parse_config(config_object, exclude=('key',))` instead."
+            ),
+        }
+        for deprecated_kwarg, msg in deprecated_kwargs.items():
+            if kwargs.get(deprecated_kwarg):
+                self.deprecated_features_used[deprecated_kwarg] = msg
 
         init_config = parse_config(
-            init_config, include=config_include_keys, exclude=config_exclude_keys
+            init_config,
+            include=kwargs.pop("config_include_keys", None),
+            exclude=kwargs.pop("config_exclude_keys", None),
         )
 
         # merge config with sweep or sagemaker (or config file)
@@ -231,7 +228,7 @@ class _WandbInit:
         init_settings = {
             key: kwargs[key]
             for key in ["anonymous", "force", "mode", "resume"]
-            if kwargs.get(key, None) is not None
+            if kwargs.get(key) is not None
         }
         if init_settings:
             settings.update(init_settings, source=Source.INIT)
@@ -242,6 +239,7 @@ class _WandbInit:
                 force=kwargs.pop("force", None),
                 _disable_warning=True,
                 _silent=settings.quiet or settings.silent,
+                _entity=kwargs.get("entity") or settings.entity,
             )
 
         # apply updated global state after login was handled
@@ -261,14 +259,7 @@ class _WandbInit:
             settings.update({"save_code": False}, source=Source.INIT)
 
         # TODO(jhr): should this be moved? probably.
-        time_stamp: float = time.time()
-        settings.update(
-            {
-                "_start_time": time_stamp,
-                "_start_datetime": datetime.datetime.fromtimestamp(time_stamp),
-            },
-            source=Source.INIT,
-        )
+        settings._set_run_start_time(source=Source.INIT)
 
         if not settings._noop:
             self._log_setup(settings)
@@ -289,8 +280,8 @@ class _WandbInit:
     def _enable_logging(self, log_fname, run_id=None):
         """Enables logging to the global debug log.
 
-        This adds a run_id to the log, in case of muliple processes on the same machine.
-        Currently there is no way to disable logging after it's enabled.
+        This adds a run_id to the log, in case of multiple processes on the same machine.
+        Currently, there is no way to disable logging after it's enabled.
         """
         handler = logging.FileHandler(log_fname)
         handler.setLevel(logging.INFO)
@@ -525,6 +516,7 @@ class _WandbInit:
 
         manager = self._wl._get_manager()
         if manager:
+            logger.info("setting up manager")
             manager._inform_init(settings=self.settings, run_id=self.settings.run_id)
 
         backend = Backend(settings=self.settings, manager=manager)
@@ -536,17 +528,16 @@ class _WandbInit:
 
         # resuming needs access to the server, check server_status()?
 
-        run = Run(
-            config=self.config, settings=self.settings, sweep_config=self.sweep_config
-        )
+        run = Run(config=self.config, settings=self.settings)
 
         # probe the active start method
         active_start_method: Optional[str] = None
         if self.settings.start_method == "thread":
             active_start_method = self.settings.start_method
         else:
-            get_start_fn = getattr(backend._multiprocessing, "get_start_method", None)
-            active_start_method = get_start_fn() if get_start_fn else None
+            active_start_method = getattr(
+                backend._multiprocessing, "get_start_method", lambda: None
+            )()
 
         # Populate initial telemetry
         with telemetry.context(run=run, obj=self._init_telemetry_obj) as tel:
@@ -580,7 +571,7 @@ class _WandbInit:
 
             tel.env.maybe_mp = _maybe_mp_process(backend)
 
-            # fixme: detected issues with settings
+            # todo: detected issues with settings.
             if self.settings.__dict__["_Settings__preprocessing_warnings"]:
                 tel.issues.settings__preprocessing_warnings = True
             if self.settings.__dict__["_Settings__validation_warnings"]:
@@ -594,6 +585,14 @@ class _WandbInit:
             else:
                 run._label_probe_main()
 
+        for deprecated_feature, msg in self.deprecated_features_used.items():
+            warning_message = f"`{deprecated_feature}` is deprecated. {msg}"
+            deprecate(
+                field_name=getattr(Deprecated, "init__" + deprecated_feature),
+                warning_message=warning_message,
+                run=run,
+            )
+
         logger.info("updated telemetry")
 
         run._set_library(self._wl)
@@ -625,8 +624,12 @@ class _WandbInit:
                     f"Starting a new run with run id {run.id}."
                 )
         else:
-            logger.info("communicating run to backend with 30 second timeout")
-            run_result = backend.interface.communicate_run(run, timeout=30)
+            logger.info(
+                f"communicating run to backend with {self.settings.init_timeout} second timeout"
+            )
+            run_result = backend.interface.communicate_run(
+                run, timeout=self.settings.init_timeout
+            )
 
             error_message: Optional[str] = None
             if not run_result:
@@ -642,11 +645,11 @@ class _WandbInit:
                 error_message = run_result.error.message
             if error_message:
                 logger.error(f"encountered error: {error_message}")
-
-                # Shutdown the backend and get rid of the logger
-                # we don't need to do console cleanup at this point
-                backend.cleanup()
-                self.teardown()
+                if not manager:
+                    # Shutdown the backend and get rid of the logger
+                    # we don't need to do console cleanup at this point
+                    backend.cleanup()
+                    self.teardown()
                 raise UsageError(error_message)
             assert run_result and run_result.run
             if run_result.run.resumed:
@@ -675,6 +678,7 @@ class _WandbInit:
         # put artifacts in run config here
         # since doing so earlier will cause an error
         # as the run is not upserted
+        run._populate_sweep_or_launch_config(self.sweep_config)
         for k, v in self.init_artifact_config.items():
             run.config.update({k: v}, allow_val_change=True)
 
@@ -872,7 +876,7 @@ def init(
             "production". It's easy to add and remove tags in the UI, or filter
             down to just runs with a specific tag.
         name: (str, optional) A short display name for this run, which is how
-            you'll identify this run in the UI. By default we generate a random
+            you'll identify this run in the UI. By default, we generate a random
             two-word name that lets you easily cross-reference runs from the
             table to charts. Keeping these run names short makes the chart
             legends and tables easier to read. If you're looking for a place to
@@ -882,14 +886,14 @@ def init(
             ran this run.
         dir: (str, optional) An absolute path to a directory where metadata will
             be stored. When you call `download()` on an artifact, this is the
-            directory where downloaded files will be saved. By default this is
+            directory where downloaded files will be saved. By default, this is
             the `./wandb` directory.
         resume: (bool, str, optional) Sets the resuming behavior. Options:
             `"allow"`, `"must"`, `"never"`, `"auto"` or `None`. Defaults to `None`.
             Cases:
             - `None` (default): If the new run has the same ID as a previous run,
                 this run overwrites that data.
-            - `"auto"` (or `True`): if the preivous run on this machine crashed,
+            - `"auto"` (or `True`): if the previous run on this machine crashed,
                 automatically resume it. Otherwise, start a new run.
             - `"allow"`: if id is set with `init(id="UNIQUE_ID")` or
                 `WANDB_RUN_ID="UNIQUE_ID"` and it is identical to a previous run,
@@ -926,7 +930,7 @@ def init(
         mode: (str, optional) Can be `"online"`, `"offline"` or `"disabled"`. Defaults to
             online.
         allow_val_change: (bool, optional) Whether to allow config values to
-            change after setting the keys once. By default we throw an exception
+            change after setting the keys once. By default, we throw an exception
             if a config value is overwritten. If you want to track something
             like a varying learning rate at multiple times during training, use
             `wandb.log()` instead. (default: `False` in scripts, `True` in Jupyter)
@@ -1009,7 +1013,7 @@ def init(
             # TODO(jhr): figure out how to make this RunDummy
             run = None
     except UsageError as e:
-        wandb.termerror(str(e))
+        wandb.termerror(str(e), repeat=False)
         raise
     except KeyboardInterrupt as e:
         assert logger
